from functools import wraps
from datetime import datetime
from posixpath import join
from pydantic import BaseModel

from max_ard.session import get_user_session, get_self, ARD_HOSTNAME, ARD_DEV_BRANCH


class Usage(BaseModel):
    limit_sqkm: float
    fresh_ordered_sqkm: float
    standard_ordered_sqkm: float
    training_ordered_sqkm: float
    total_ordered_sqkm: float
    available_sqkm: float


    def __str__(self):
        output = {}
        for k,v in self.dict().items():
            if v == -1:
                output[k] = "unlimited"
            else:
                output[k] = f'{v:.1f} sqkm'
        return f'''
 Data Usage
 ----------

  Account limit: {output['limit_sqkm']}

  Imagery ordered: {output['total_ordered_sqkm']}
  ├ Fresh (< 90 days): {output['fresh_ordered_sqkm']}
  ├ Standard (90 days - 3 years): {output['standard_ordered_sqkm']}
  └ Training (> 3 years): {output['training_ordered_sqkm']}

  Remaining balance: {output['available_sqkm']}
  
  '''


class AccountManager():

    def __init__(self, account_id=None, session=None) -> None:
        # we'll be able to pull the account id from the user self endpoint
        # and do this automatically
        if not session:
            session = get_user_session()
        self.session = session
        if not account_id:
            account_id = get_self(self.session)['account_id']
        self.account_id = account_id
        self.base_url = f'https://{ARD_HOSTNAME}/admin{ARD_DEV_BRANCH}/account/{account_id}'
        self.properties = self.get('')['account']

    def api_helper(func):
        @wraps(func)
        def wrapper(self, url, **kwargs):
            url = join(self.base_url, url)
            r = func(self, url, **kwargs)
            # use func.__name__ to enhance error message or generate new exception
            r.raise_for_status()
            if r.status_code == 204:
                return None
            else:
                return r.json()
        return wrapper

    @api_helper
    def get(self, url):
        return self.session.get(url)
    
    @api_helper
    def put(self, url, json=None):
        return self.session.put(url, json=json) 

    @api_helper
    def post(self, url, json=None):
        return self.session.post(url, json=json) 

    @api_helper
    def delete(self, url):
        return self.session.delete(url) 


    ########
    # Usage
    ########

    def _validate_dates(self, *args):
        for date in args:
            if date is not None:
                try:
                    datetime.strptime(date, '%Y-%m-%d')
                except ValueError:
                    raise ValueError('Dates must be YYYY-MM-DD')
        
    def get_account_usage(self, start_date=None, end_date=None):

        url = f'https://{ARD_HOSTNAME}/usage{ARD_DEV_BRANCH}/account/{self.account_id}'

        self._validate_dates(start_date, end_date)

        params = {
            'start_date': start_date,
            'end_date': end_date
        }

        r = self.session.get(url, params=params)
        r.raise_for_status()
        return Usage(**r.json()['usage'])

    def get_user_usage(self, username=None, start_date=None, end_date=None):
        if not username:
            username = get_self(self.session)['username']

        url = f'https://{ARD_HOSTNAME}/usage{ARD_DEV_BRANCH}/user/{username}'

        params = {
            'start_date': start_date,
            'end_date': end_date
        }

        r = self.session.get(url, params=params)
        r.raise_for_status()
        return Usage(**r.json()['usage'])

   