import os
from time import strptime
from getpass import getpass
import click
from click import exceptions
from max_ard.commands.storage import storage
from max_ard.commands.grid import cell
from max_ard.commands.tile import tile
from max_ard.commands.select import select
from max_ard.commands.grid import cell
from max_ard.commands.order import order
from max_ard.admin import AccountManager
from max_ard.session import write_config, get_user_session, get_self, read_token
from max_ard.session import ARD_HOSTNAME, ARD_DEV_BRANCH


def check_access():
    get_self(get_user_session())
    return True

@click.group()
def max_ard():
    '''Maxar ARD tools'''
    pass


@click.command()
@click.option('--username', help='ARD username')
@click.option('--password', help='ARD password')
def login(username, password):
    ''' Creates credential file using passed options --username and --password and tests access 
        to the ARD system. If run without options, 
        will only check if existing credential file at ~/.ard-config contains valid credentials '''

    if username or password:
        click.echo("... Creating config file ~/.ard-config")
        write_config(username, password)
    else:
        if not os.path.exists(os.path.expanduser('~/.ard-config')):
            click.echo("No credential file exists - let's make a new one.")
            username = input('ARD account name? ')
            password = getpass(prompt='ARD account password? ')
            write_config(username, password)
    try:
        click.echo('... Checking access')
        check_access()
        click.secho('ARD access confirmed', fg='green')
    except Exception as ex:
        click.secho(f'There was an error logging in: {ex}', fg='red', err=True)

@click.command()
def token():
    ''' Returns the current token. 

        Handy for substitution in bash commands like curl:

            \b
            curl -H 'Accept: application/json' \\
                 -H "Authorization: Bearer $(max_ard token)" \\
                 https://hostname/api/myresource
    '''

    try:
        check_access()
    except:
        raise
    click.echo(read_token())


@click.command()
@click.option('--user', help='ARD username')
@click.option('--account', help='ARD account')
@click.option('--start-date', 'start_date', help='Start date YYYY-MM-DD')
@click.option('--end-date', 'end_date', help='End date YYYY-MM-DD')
def usage(user, account, start_date, end_date):
    self = get_self(get_user_session())
    while not any([user, account, start_date, end_date]):
        source = input('Get usage for an [a]ccount or [u]ser? ')
        if source.lower() not in ['a', 'u']: 
            click.secho('Please press "a" for account or "u" for user', fg='red')
            continue
        if source.lower() == 'a':
            account = input('Account number [current account]? ')
            if not account:
                account = self['account_id']
        else:
            user = input('User ID [current user]? ')
            if not user:
                user = self['username']
            account = input(f'Temporary: account number for user {user}? ')
        start_date = input('Start date YYYY-MM-DD [none]? ')
        end_date = input('End date YYYY-MM-DD [none]? ')


    manager = AccountManager(account)

    try:
        if user:
            usage = manager.get_user_usage(user, start_date, end_date)
        else:
            usage = manager.get_account_usage(start_date, end_date)
    except Exception as e:
        click.secho(f'Error: {e}', err=True, fg='red',)
        exit(1)

    click.echo(str(usage))



max_ard.add_command(login)
max_ard.add_command(token)
max_ard.add_command(usage)


max_ard.add_command(storage)
max_ard.add_command(cell)
#max_ard.add_command(tile)
max_ard.add_command(cell)
max_ard.add_command(select)
max_ard.add_command(order)

if __name__ == '__main__':
    max_ard()