import json
import click
import webbrowser
import pprint
from tempfile import NamedTemporaryFile
from max_ard.order import Order
import sys
from collections import defaultdict
from max_ard.ard_collection import ARDCollection
from max_ard.outputs.qlr_format import QlrDoc
from max_ard.exceptions import BadARDRequest

@click.group()
def order():
    ''' Tools for ordering ARD products '''
    pass


@click.command()
@click.argument('order_id', required=True)
@click.option('--format', help='Output the order status in other formats: `raw` for the full response JSON')
def status(order_id, format):
    ''' Gets the status of an order given the ORDER_ID '''

    try:
        s = Order.from_id(order_id)
    except Exception as e:
        click.echo(str(e), err=True)
        sys.exit()
    if format == 'raw':
        click.echo(s.response.dict())
    else:
        click.echo(s.state)


@click.command()
@click.argument('order_id', required=True)
@click.option('--format', help='Output the order in other formats: "qlr" for QGIS layer')
def describe(order_id, format):
    ''' Outputs order details '''
    try:
        order = Order.from_id(order_id)
    except Exception as e:
        click.secho(str(e), err=True, fg='red')
        sys.exit()
    pp = pprint.PrettyPrinter(indent=4)
    if format == 'qlr':
        c = ARDCollection.from_order(order)
        click.echo(QlrDoc(c))
        exit()
    if not order.finished:
        click.echo('Order has not finished running')
        click.echo('*******************')
        click.echo('Request details:')
        pp.pprint(order.response.order)
        exit()
    c = ARDCollection.from_order(order)
    click.echo(f'getting Order ID {order_id}')
    click.echo('*******************')
    click.echo('Order details:')
    pp.pprint(order.response.order)
    click.echo('*******************')
    if format is None:
        def S(i):
            s = 's'
            try: 
                i = len(i)
            except:
                pass
            if i == 1:
                s = ''
            return s
        try:
            count = len(c.acq_ids)
        except PermissionError:
            click.secho(f'Your credentials for accessing the order are invalid or expired', err=True, fg='red')
            exit()
        if count == 0:
            click.echo('No ARD tiles found')
            exit()
        click.echo(f'{count} acquisition{S(count)}')
        click.echo(f'{len(c.tiles)} tile{S(c.tiles)}')
        click.echo(f'Earliest date: {c.start_date}')
        click.echo(f'Latest date: {c.end_date}')
        zones = ','.join([str(z) for z in c.zones])
        click.echo(f'UTM zone{S(c.zones)}: {zones}')
    else:
        click.secho('QGIS Layer (--format qlr) is the only available format')
    click.echo('\n\n')

class NumericType(click.ParamType):
    name = "numeric"

    def convert(self, value, param, ctx):
        try:
            return float(value)
        except TypeError:
            self.fail(
                "expected string for int() conversion, got "
                f"{value!r} of type {type(value).__name__}",
                param,
                ctx,
            )
        except ValueError:
            self.fail(f"{value!r} is not a value that can be converted to a float", param, ctx)

NUM_TYPE = NumericType()


@click.command()
@click.option('--destination', required=True, help='Destination of the tiles: an S3 bucket and optional prefix')
@click.option('--select_id', help='Order the results of Select with the Select ID')
@click.option('--acquisition_id', 'acquisitions', multiple=True, help='Order a given acquisition by acquisition ID, can be provided multiple times.' + \
    'If provided with a Select ID will only generate tiles for the provided IDs.')
@click.option('--intersects', help='Only generate tiles that intesect this geometry. Accepts WKT only.')
@click.option('--role_arn', help='A trusted Role ARN for the writer to assume so it can write tiles.')
@click.option('--dry-run', is_flag=True, help='Submit the order for basic validation without processing the order')
@click.option('--bbox', nargs=4, type=NUM_TYPE, help='Like --intersects, but takes a bounding box in the form --bbox XMIN YMIN XMAX YMAX')
@click.option('--bba', type=bool, default=False, help='Use Bundle Block Adjustment to align images within this order. See docs for limitations and best practices for using BBA.')
@click.option('--add-email', 'emails', multiple=True, help='Add an email address to receive order notifications. Can be used multiple times to add more than one email address.')
@click.option('--add-sns', 'topics', multiple=True, help='Add an AWS SNS topic to send nofications to. Can be used multiple times to add more than one topic.')
def submit(**kwargs):
    ''' Submit an ARD Order '''

    emails = kwargs.pop('emails', [])
    topics = kwargs.pop('topics', [])
    s = Order(**kwargs)
    if not emails and topics:
        if input("No notifications supplied, do you still want to order?") != "y":
            exit()
    for email in emails:
        s.add_email_notification(email)
    for topic in topics:
        s.add_sns_notification(topic)
    click.echo('Request payload:')
    click.echo(s.request.to_payload())

    if kwargs['dry_run']:
        click.echo('Checking order in dry-run mode...')
        try:
            s.submit()
        except BadARDRequest as e:
            click.secho(f'There was an error in your request: {e}', err=True, fg='red')
            exit()
        click.secho('Dry run successful', fg='green')
    else:
        if input("Are you sure you want to order? (y/n) ") != "y":
            exit()
        try:
            s.submit()
            click.echo(f'Order {s.order_id} has been submitted')
            click.echo(f'Run `max-ard order status {s.order_id}` to check the status')
            click.echo('When the status is "SUCCEEDED" you can: ')
            click.echo(f'Run `max-ard order describe {s.order_id}` to see a basic overview of the results')
            click.echo('Run `max-ard order` to see all the `order` commands')
        except BadARDRequest as e:
            click.secho(f'There was an error in your request: {e}', err=True, fg='red')

        


order.add_command(status)
order.add_command(describe)
order.add_command(submit)
